/*------------------------------------

	flash ROM writer firmware (3/3)
	
	2010.09.25 by Koichi Nishida

-------------------------------------*/

/*
This is a part of the firmware for the flash ROM writer by Nishida Radio.

Copyright (C) 2010 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

	.include "tn2313def.inc"

	.cseg

	rjmp	VECT_RESET
	rjmp	VECT_INT0
	rjmp	VECT_INT1
	rjmp	VECT_TIMER1_CAPT
	rjmp	VECT_TIMER1_COMPA
	rjmp	VECT_TIMER1_OVF
	rjmp	VECT_TIMER0_OVF
	rjmp	VECT_USART0_RX
	rjmp	VECT_USART0_UDRE
	rjmp	VECT_USART0_TX
	rjmp	VECT_ANALOG_COMP
	rjmp	VECT_PCINT
	rjmp	VECT_TIMER1_COMPB
	rjmp	VECT_TIMER0_COMPA
	rjmp	VECT_TIMER0_COMPB
	rjmp	VECT_USI_START
	rjmp	VECT_USI_OVERFLOW
	rjmp	VECT_EE_READY
	rjmp	VECT_WDT_OVERFLOW

VECT_INT0:
VECT_INT1:
VECT_TIMER1_CAPT:
VECT_TIMER1_COMPA:
VECT_TIMER1_OVF:
VECT_TIMER0_OVF:
VECT_USART0_RX:
VECT_USART0_UDRE:
VECT_USART0_TX:
VECT_ANALOG_COMP:
VECT_PCINT:
VECT_TIMER1_COMPB:
VECT_TIMER0_COMPA:
VECT_TIMER0_COMPB:
VECT_USI_START:
VECT_USI_OVERFLOW:
VECT_EE_READY:
VECT_WDT_OVERFLOW:
	reti

VECT_RESET:
	ldi		r16,0
	out		PORTA,r16
	out		PORTB,r16
	ldi		r16,0b00110000
	out		PORTD,r16

	ldi		r16,0b00000011
	out		DDRA,r16
	ldi		r16,0b11111111
	out		DDRB,r16
	ldi		r16,0b00110000
	out		DDRD,r16

	; UART 19200 baud
	ldi		r16,0
	out		UBRRH,r16
	ldi		r16,0
	out		UBRRL, r16
	ldi		r16,(1<<RXEN)
	out		UCSRB,r16
	ldi		r16,(1<<USBS)|(3<<UCSZ0)
	out		UCSRC,r16

	rjmp	CLEAR

LOOP:

USART_Receive:
	sbis	UCSRA, RXC
	rjmp	USART_Receive
	in		r16,UDR
	
	cpi		r16,0x55
	breq	AD55
	cpi		r16,0xaa
	breq	ADAA
	cpi		r16,0
	breq	VALID
	cpi		r16,1
	breq	WE
	cpi		r16,2
	breq	WD
	cpi		r16,3
	breq	OE
	cpi		r16,4
	breq	OD
	cpi		r16,5
	breq	CLEAR
	cpi		r16,6
	breq	INCR

	rjmp	LOOP

AD55:
	ldi		r16,0x55
	out		PORTB,r16
	rjmp	LOOP

ADAA:
	ldi		r16,0x2a
	out		PORTB,r16
	rjmp	LOOP

WE:
	cbi		PORTD,4
	rjmp	LOOP

WD:
	sbi		PORTD,4
	rjmp	LOOP

OE:
	cbi		PORTD,5
	rjmp	LOOP

OD:
	sbi		PORTD,5
	rjmp	LOOP

CLEAR:
	ldi		r18,0
	ldi		r19,0
	ldi		r20,0
VALID:
	out		PORTB,r19
	out		PORTA,r20
	rjmp	LOOP

INCR:
	ldi		r21,0
	ldi		r22,1
	add		r18,r22
	adc		r19,r21
	adc		r20,r21
	out		PORTB,r19
	out		PORTA,r20
	rjmp	LOOP
